﻿Imports Tsql
Imports System.Globalization

''' -----------------------------------------------------------------------------
''' <summary>
''' Provides functions for manipulating database field names.
''' </summary>
''' <remarks>
''' Use the functions in the class to work with database field names.
''' </remarks>
''' -----------------------------------------------------------------------------
Public NotInheritable Class Field

#Region " Declarations "
    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Prevents initialization of a new instance of a <c>Field</c> object.
    ''' </summary>
    ''' <remarks>
    ''' </remarks>
    ''' -----------------------------------------------------------------------------
    Private Sub New()

    End Sub
#End Region

#Region " Properties "

#End Region

#Region " Methods "
    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Gets a quoted database field name.
    ''' </summary>
    ''' <param name="strFieldName">The field name to retrieve a quoted field name 
    ''' for.</param>
    ''' <returns>A quoted database field name in which " characters enclose the 
    ''' field name.</returns>
    ''' <remarks>
    ''' </remarks>
    ''' -----------------------------------------------------------------------------
    Public Overloads Shared Function GetQuotedFieldName(ByVal strFieldName As String) As String
        Return GetQuotedFieldName(strFieldName, String.Empty, Keyword.OpenQuote, Keyword.CloseQuote)
    End Function

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Gets a quoted database field name.
    ''' </summary>
    ''' <param name="strFieldName">The field name to retrieve a quoted field name 
    ''' for.</param>
    ''' <param name="strPrefix">A string to prefix the field name with.</param>
    ''' <returns>A quoted database field name in which " characters enclose the 
    ''' field name.</returns>
    ''' <remarks>
    ''' The returned quoted field name is prefixed with <paramref name="strPrefix" />.
    ''' </remarks>
    ''' -----------------------------------------------------------------------------
    Public Overloads Shared Function GetQuotedFieldName(ByVal strFieldName As String, ByVal strPrefix As String) As String
        Return GetQuotedFieldName(strFieldName, strPrefix, Keyword.OpenQuote, Keyword.CloseQuote)
    End Function

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Gets a quoted database field name.
    ''' </summary>
    ''' <param name="strFieldName">The field name to retrieve a quoted field name 
    ''' for.</param>
    ''' <param name="strPrefix">A string to prefix the field name with.</param>
    ''' <param name="chrOpenQuote">The to use to quote the field name.</param>
    ''' <param name="chrCloseQuote">The to use to quote the field name.</param>
    ''' <returns>A quoted database field name in which <paramref name="chrQuote"/> 
    ''' encloses the field name.</returns>
    ''' <remarks>
    ''' The returned quoted field name is prefixed with <paramref name="strPrefix"/>.
    ''' </remarks>
    ''' -----------------------------------------------------------------------------
    Public Overloads Shared Function GetQuotedFieldName(ByVal strFieldName As String, ByVal strPrefix As String, ByVal chrOpenQuote As Char, ByVal chrCloseQuote As Char) As String
        If IsNothing(strFieldName) Then
            Throw New ArgumentNullException("strFieldName")
        End If

        If (strFieldName.StartsWith(chrOpenQuote) And strFieldName.EndsWith(chrCloseQuote)) Then
            If (strPrefix = String.Empty) Then
                Return strFieldName
            Else
                Return strFieldName.Insert(1, strPrefix)
            End If
        Else
            Return String.Format("{0}{2}{3}{1}", chrOpenQuote, chrCloseQuote, strPrefix, strFieldName)
        End If

    End Function

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Gets an unquoted field name.
    ''' </summary>
    ''' <param name="strFieldName">The field name to get an unquoted field name from.
    ''' </param>
    ''' <returns>An unquoted field name.</returns>
    ''' <remarks>
    ''' </remarks>>
    ''' -----------------------------------------------------------------------------
    Public Overloads Shared Function GetFieldName(ByVal strFieldName As String) As String
        If IsNothing(strFieldName) Then
            Throw New ArgumentNullException("strFieldName")
        End If

        If (strFieldName.StartsWith(Keyword.OpenQuote) _
            And strFieldName.EndsWith(Keyword.CloseQuote)) _
        Then
            Return strFieldName.Substring(1, strFieldName.Length - 2)
        Else
            Return strFieldName
        End If

    End Function

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Gets an unquoted field name.
    ''' </summary>
    ''' <param name="strFieldName">The field name to get an unquoted field name from.
    ''' </param>
    ''' <param name="strPrefix">A string to prefix the field name with.</param>
    ''' <returns>An unquoted field name.</returns>
    ''' <remarks>
    ''' The returned unquoted field name is prefixed with <paramref name="strPrefix"/>.
    ''' </remarks>
    ''' -----------------------------------------------------------------------------
    Public Overloads Shared Function GetFieldName(ByVal strFieldName As String, ByVal strPrefix As String) As String
        If IsNothing(strFieldName) Then
            Throw New ArgumentNullException("strFieldName")
        End If

        If (strFieldName.StartsWith(Keyword.OpenQuote) _
            And strFieldName.EndsWith(Keyword.CloseQuote)) _
        Then
            Return strPrefix + strFieldName.Substring(1, strFieldName.Length - 2)
        Else
            Return strPrefix + strFieldName
        End If

    End Function

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Gets an SQL field name alias string.
    ''' </summary>
    ''' <param name="strFieldName">The field name to get an alias string for.</param>
    ''' <param name="strAlias">The alias that should be use for the field.</param>
    ''' <returns>A AQL field name alias string.</returns>
    ''' <remarks>This function returns an a field name alias string that can be used 
    ''' in an SQL select statement as part of a field selection list.
    ''' </remarks>
    ''' -----------------------------------------------------------------------------
    Public Shared Function GetSqlAliasString(ByVal strFieldName As String, ByVal strAlias As String) As String
        Return String.Concat(GetQuotedFieldName(strFieldName), Keyword.As, GetQuotedFieldName(strAlias))
    End Function
#End Region

End Class
