﻿Imports Tsql
Imports System.ComponentModel
Imports System.Data.Odbc
Imports System.Globalization

''' -----------------------------------------------------------------------------
''' <summary>
''' Provides functions for building and manipulating a database group by
''' string.
''' </summary>
''' <remarks>Provides functions for generating a human readable group by as
''' well as an SQL sort criteria string.
''' </remarks>
''' -----------------------------------------------------------------------------
<Serializable()> _
Public Class GroupBy

#Region " Fields "
    Private _columnName As String
    Private _selectFields As FieldCollection
    Private _sortIndicator As SortIndicator
    Private _textFormat As String = "[ColumnName]: [Value] (Items: [Count])"
#End Region

#Region " Constructors "
    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Creates a new <c>GroupBy</c> object.
    ''' </summary>
    ''' <remarks>Creates a new empty <c>GroupBy</c> object.
    ''' </remarks>
    ''' -----------------------------------------------------------------------------
    Public Sub New()
        ColumnName = String.Empty
        SortIndicator = SortIndicator.None
    End Sub

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Creates a new <c>GroupBy</c> object from a group by string.
    ''' </summary>
    ''' <param name="strGroupBy">A group by string.</param>
    ''' <remarks>
    ''' <p>Creates a new <c>GroupBy</c> object and initialises it with the 
    ''' grouping specified in the <paramref name="strGroupBy"/> parameter.
    ''' </p>
    ''' <p>The <paramref name="strGroupBy"/> parameter should be in the format 
    ''' <c><i>fieldname</i> <b>(</b> { <b>ascending</b> | <b>descending</b> } <b>)
    ''' </b></c></p>
    ''' </remarks>
    ''' -----------------------------------------------------------------------------
    Public Sub New(ByVal strGroupBy As String)
        If IsNothing(strGroupBy) Then
            Throw New ArgumentNullException("strGroupBy")
        End If

        If (strGroupBy.EndsWith(" (ascending)")) Then
            Me._columnName = Field.GetFieldName(strGroupBy.Substring(0, strGroupBy.Length - 12))
            Me._sortIndicator = SortIndicator.Ascending
        ElseIf (strGroupBy.EndsWith(" ASC")) Then
            Me._columnName = Field.GetFieldName(strGroupBy.Substring(0, strGroupBy.Length - 4))
            Me._sortIndicator = SortIndicator.Ascending
        ElseIf (strGroupBy.EndsWith(" ASCENDING")) Then
            Me._columnName = Field.GetFieldName(strGroupBy.Substring(0, strGroupBy.Length - 10))
            Me._sortIndicator = SortIndicator.Ascending
        ElseIf (strGroupBy.EndsWith(" (descending)")) Then
            Me._columnName = Field.GetFieldName(strGroupBy.Substring(0, strGroupBy.Length - 13))
            Me._sortIndicator = SortIndicator.Descending
        ElseIf (strGroupBy.EndsWith(" DESC")) Then
            Me._columnName = Field.GetFieldName(strGroupBy.Substring(0, strGroupBy.Length - 5))
            Me._sortIndicator = SortIndicator.Descending
        ElseIf (strGroupBy.EndsWith(" DESCENDING")) Then
            Me._columnName = Field.GetFieldName(strGroupBy.Substring(0, strGroupBy.Length - 11))
            Me._sortIndicator = SortIndicator.Descending
        Else
            Me._columnName = Field.GetFieldName(strGroupBy)
            Me._sortIndicator = SortIndicator.Ascending
        End If

    End Sub

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Creates a new <c>GroupBy</c> object from a field name and sort 
    ''' direction.
    ''' </summary>
    ''' <param name="strColumnName">The name of a field.</param>
    ''' <param name="fSortIndicator">A <c>SortIndicator</c> sort direction indicator 
    ''' flag.</param>
    ''' <remarks>Creates a new <c>GroupBy</c> object and initialises it with the
    ''' given sort field and sort direction.
    ''' </remarks>
    ''' -----------------------------------------------------------------------------
    Public Sub New(ByVal strColumnName As String, ByVal fSortIndicator As SortIndicator)
        MyBase.New()
        Me._columnName = strColumnName
        Me._sortIndicator = fSortIndicator
    End Sub
#End Region

#Region " Properties "
    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Gets or sets the name of the group column/field in a <c>GroupBy</c>
    ''' object.
    ''' </summary>
    ''' <returns>The name of the group column/field.</returns>
    ''' -----------------------------------------------------------------------------
    Public Property ColumnName() As String
        Get
            Return _columnName
        End Get
        Set(ByVal newColumnName As String)
            If IsNothing(newColumnName) Then
                Throw New ArgumentNullException("newColumnName")
            End If
            _columnName = Field.GetFieldName(newColumnName.Trim())
        End Set
    End Property

    ''' <summary>
    ''' Gets a collection of field names to select for the group.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks>By default the group field value and count of items are retrieved.
    ''' To use additional data to format the group header, add the required field
    ''' names to this collection.</remarks>
    Public ReadOnly Property SelectFields() As FieldCollection
        Get
            If (Me._selectFields Is Nothing) Then
                Me._selectFields = New FieldCollection()
            End If
            Return Me._selectFields
        End Get
    End Property

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Gets or sets a value indicating the sort direction of a <c>GroupBy</c>
    ''' object's column/field.
    ''' </summary>
    ''' <value>A <c>SortIndicator</c> value indicating the direction in which the 
    ''' <c>GroupBy</c> object's column/field should be sorted.</value>
    ''' -----------------------------------------------------------------------------
    <DefaultValue(SortIndicator.None)> _
    Public Property SortIndicator() As SortIndicator
        Get
            Return _sortIndicator
        End Get
        Set(ByVal newSortIndicator As SortIndicator)
            _sortIndicator = newSortIndicator
        End Set
    End Property

    ''' <summary>
    ''' The format string for the group header text.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property TextFormat() As String
        Get
            Return Me._textFormat
        End Get
        Set(ByVal value As String)
            Me._textFormat = value
        End Set
    End Property
#End Region

#Region "Methods"
    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Creates a new <c>GroupBy</c> object from an SQL group criteria string.
    ''' </summary>
    ''' <param name="strGroupBy">An SQL group criteria string.</param>
    ''' <remarks>
    ''' <p>Creates a new <c>GroupBy</c> object and initialises it with the 
    ''' group criteria specified in the <paramref name="strGroupBy"/> parameter.
    ''' </p>
    ''' <p>The <paramref name="strGroupBy"/> parameter should be in the format 
    ''' <c><i>fieldname</i> { <b>ASC</b> | <b>DESC</b> }</c></p>
    ''' </remarks>
    ''' -----------------------------------------------------------------------------
    Public Sub FromSql(ByVal strGroupBy As String)
        If IsNothing(strGroupBy) Then
            Throw New ArgumentNullException("strGroupBy")
        End If

        Dim nChar As Integer = strGroupBy.LastIndexOf(" "c)
        If (nChar = -1) Then
            Throw New ArgumentException("Invalid GroupBy String.")
        End If

        ' Get the unquoted column name
        ColumnName = Left$(strGroupBy, nChar)

        If (Mid$(strGroupBy, nChar + 1, Len(strGroupBy) - nChar).Equals(Keyword.Asc)) Or (strGroupBy.Contains(Keyword.Ascending)) Then
            SortIndicator = SortIndicator.Ascending
        Else
            SortIndicator = SortIndicator.Descending
        End If
    End Sub

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Gets a human readable group criteria string from a <c>GroupBy</c> object.
    ''' </summary>
    ''' <returns>A group criteria string.</returns>
    ''' <remarks>
    ''' <p>If the <c>GroupBy</c> object is empty, returns an <c>Empty</c> 
    ''' string.</p>
    ''' <p>The returned string is in the format <c><i>columnname</i> <b>(</b> { <b>
    ''' ascending</b> | <b>descending</b> } <b>)</b></c></p>
    ''' </remarks>
    ''' -----------------------------------------------------------------------------
    Public Overrides Function ToString() As String
        If (ColumnName.Length = 0) Then
            Return String.Empty
        End If

        If (SortIndicator = SortIndicator.None) Then
            Return String.Empty
        End If

        Return String.Format("{0} ({1})", ColumnName, SortIndicator.ToString().ToLower())

    End Function

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Gets an SQL group criteria string from a <c>GroupBy</c> object.
    ''' </summary>
    ''' <returns>A group criteria string.</returns>
    ''' <remarks>
    ''' <p>If the <c>GroupBy</c> object is empty, returns an <c>Empty</c> 
    ''' string.</p>
    ''' <p>The returned string is in the format <c><i>columnname</i>  [ { <b>ASC</b> 
    ''' | <b>DESC</b> } ]</c></p>
    ''' </remarks>
    ''' -----------------------------------------------------------------------------
    Public Overridable Function ToSql() As String

        If (ColumnName.Length = 0) Then
            Return String.Empty
        End If

        If (SortIndicator = SortIndicator.None) Then
            Return String.Empty
        End If

        Return String.Format("{0}{1}", Field.GetQuotedFieldName(ColumnName), IIf(SortIndicator.Equals(SortIndicator.Ascending), Keyword.Asc, Keyword.Desc))

    End Function

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Gets an SQL group criteria string from a <c>GroupBy</c> object.
    ''' </summary>
    ''' <param name="cnn">An open connection to the database.</param>
    ''' <returns>A group criteria string.</returns>
    ''' <remarks>
    ''' <p>If the <c>GroupBy</c> object is empty, returns an <c>Empty</c> 
    ''' string.</p>
    ''' <p>The returned string is in the format <c><i>columnname</i>  [ { <b>ASC</b> 
    ''' | <b>DESC</b> } ]</c></p>
    ''' </remarks>
    ''' -----------------------------------------------------------------------------
    Public Overridable Function ToSql(ByVal cnn As OdbcConnection, ByVal strTableName As String) As String

        If (ColumnName.Length = 0) Then
            Return String.Empty
        End If

        If IsNothing(cnn) Then
            Throw New ArgumentNullException("cnn")
        End If
        If IsNothing(strTableName) Then
            Throw New ArgumentNullException("strTableName")
        End If

        If (SortIndicator = SortIndicator.None) Then
            Return String.Empty
        End If

        Return String.Format("{0}{1}", Field.GetQuotedFieldName(ColumnName), IIf(SortIndicator.Equals(SortIndicator.Ascending), Keyword.Asc, Keyword.Desc))

    End Function
#End Region
End Class
